#!/usr/bin/env sh
DISPLAYNAME="⊙ Location Pin: Center on user location"
HOTKEY=x
DOC='
  Determines the user location via GPSD or Geoclue. When provided with
  no argument runs the function `droppinactivateandcenter` which both
  drops a pin on the map and centers the map. Meanwhile the function
  `droppin` can be used in isolation to only drop a pin on the map in
  pingroup 6. This script may be used synchronously or asynchronously;
  within the default config `droppin` is used asynchronously via
  `shellpipe_async`.  Note, due to deprecation of MLS; Geoclue may be
  less reliable or this script may not always work as expected recently -
  see: https://gitlab.gnome.org/GNOME/gnome-control-center/-/issues/2959
'
[ -z "$MEPO_USERPIN_ENABLED" ] && MEPO_USERPIN_ENABLED="1"
[ -z "$MEPO_TIMEOUT_GEOCLUE" ] && MEPO_TIMEOUT_GEOCLUE="10"
[ -z "$MEPO_GEOLOCATE_COORDSVIAMETHOD" ] && MEPO_GEOLOCATE_COORDSVIAMETHOD="coordsviageoclue"

coordsviageoclue() {
  supportsgeoclue || return 1

  /usr/libexec/geoclue-2.0/demos/agent > /dev/null &
  AGENTPID=$!
  LC_NUMERIC=en_US.UTF-8 /usr/libexec/geoclue-2.0/demos/where-am-i -t $MEPO_TIMEOUT_GEOCLUE |
    grep -E "Latitude|Longitude:" -m2 |
    cut -d: -f2 |
    tr -d "° " |
    tr "\n" " "
  kill -9 $AGENTPID
}

supportsgeoclue() {
  stat /usr/libexec/geoclue-2.0/demos/agent > /dev/null 2> /dev/null &&
  stat /usr/libexec/geoclue-2.0/demos/where-am-i > /dev/null 2> /dev/null
}

coordsviagpsd() {
  # On pmOS: sudo /etc/gpsd/device-hook "" ACTIVATE
  gpspipe -w -n 10 2> /dev/null | grep -m 1 TPV | jq '.lat, .lon' | tr '\n' ' '
}

iscoords() {
  echo "$1" | grep -qE '[0-9.\-]+ [0-9.\-]+'
}

getcoords() {
  # Use method set in ENV var MEPO_GEOLOCATE_COORDSVIAMETHOD
  COORDS="$($MEPO_GEOLOCATE_COORDSVIAMETHOD)"
  iscoords "$COORDS" && echo "$MEPO_GEOLOCATE_COORDSVIAMETHOD $COORDS" && return 0

  # Otherwise fallback to use geoclue otherwise
  COORDS="$(coordsviageoclue)"
  iscoords "$COORDS" && echo "coordsviageoclue $COORDS" && return 0

  # If geoclue unavailable use gpsd
  COORDS="$(coordsviagpsd)"
  iscoords "$COORDS" && echo "coordsviagpsd $COORDS" && return 0

  echo "Unable to use any avilable method to determine coordinates" >&2
  return 1
}

droppin() {
  SOURCEANDCOORDS="$(getcoords)"
  [ $? = 1 ] && return 1
  SOURCE="$(echo "$SOURCEANDCOORDS" | cut -d ' ' -f1)"
  COORDS="$(echo "$SOURCEANDCOORDS" | cut -d ' ' -f2-3)"
  LAT="$(echo "$COORDS" | cut -d " " -f1)"
  LON="$(echo "$COORDS" | cut -d " " -f2)"

  echo '[
    {
      "cmd": "pin_add",
      "args": {
        "group": 6,
        "lat": '$LAT',
        "lon": '$LON',
        "handle": "user_location"
      }
    },
    {
      "cmd": "pin_meta",
      "args": {
        "group": 6,
        "handle": "user_location",
        "key": "source",
        "value": "'$SOURCE'"
      }
    },
    {
      "cmd": "pin_meta",
      "args": {
        "group": 6,
        "handle": "user_location",
        "key": "updated_at",
        "value": "'$(date)'"
      }
    }
  ]'
}

droppinactivateandcenter() {
  BASEJSON="$(droppin)"
  [ $? = 1 ] && return 1
  ACTIVATEANDCENTERJSON='[
    {"cmd": "pin_activate", "args": {"group": 6, "handle": "user_location"}},
    {"cmd": "center_on_pin", "args": {}}
  ]'
  echo '[ '"$BASEJSON"', '"$ACTIVATEANDCENTERJSON"' ]' | jq flatten
}

main() {
  if [ "$MEPO_USERPIN_ENABLED" = 1 ]; then
    droppinactivateandcenter
  fi
}

if [ -n "$1" ]; then "$@"; else main "$@"; fi
